/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.intelliform;

import static org.assertj.core.api.Assertions.*;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import de.ozgcloud.common.test.TestUtils;
import de.ozgcloud.eingang.common.formdata.IncomingFile;
import de.ozgcloud.eingang.common.formdata.IncomingFileGroup;

class FormDataIncomingFileMapperTest {

	FormDataIncomingFileMapper mapper = new FormDataIncomingFileMapper();

	private Document document;

	@Test
	void testBuildParentNodeName() {

		assertThat(mapper.buildParentNodeName("datei_ausweis-item")).isEqualTo("datei_ausweis");
		assertThat(mapper.buildParentNodeName("datei_ausweis")).isEqualTo("datei_ausweis");
		assertThat(mapper.buildParentNodeName("datei_-item-ausweis")).isEqualTo("datei_-item-ausweis");
	}

	@Nested
	class TestEinfachesFormularMitZweiAnhaengen {

		private static final String GROUP1NAME = "Upload1";
		private static final String GROUP2NAME = "Upload2";

		List<IncomingFileGroup> attachments;

		@BeforeEach
		void init() throws ParserConfigurationException, SAXException, IOException {

			String xml = TestUtils.loadTextFile("intelliform/EinfachesFormularZweiAnhaengeXmlDaten1.xml");

			document = buildDocument(xml);

			attachments = mapper.mapAttachments(document);
		}

		@Test
		void validateAttachmentGroupCount() {

			assertThat(attachments).hasSize(2);
		}

		@Test
		void validateGroup1Name() {

			assertThat(attachments.get(0).getName()).isEqualTo(GROUP1NAME);
		}

		@Test
		void validateGroup1FilesCount() {

			assertThat(attachments.get(0).getFiles()).hasSize(1);
		}

		@Test
		void validateGroup1File() {

			IncomingFile file = attachments.get(0).getFiles().get(0);

			assertThat(file.getId()).isNotNull().hasSize(36);
			assertThat(file.getVendorId()).isEqualTo("assistants.E0FBA361C191F8B723949467AE302BEA24E4745E");
			assertThat(file.getName()).isEqualTo("Helge1.jpg");
			assertThat(file.getContentType()).isEqualTo("image/jpeg");
			assertThat(file.getSize()).isEqualTo(155251);
		}

		@Test
		void validateGroup2Name() {

			assertThat(attachments.get(1).getName()).isEqualTo(GROUP2NAME);
		}

		@Test
		void validateGroup2FilesCount() {

			assertThat(attachments.get(1).getFiles()).hasSize(1);
		}

		@Test
		void validateGroup2File() {

			IncomingFile file = attachments.get(1).getFiles().get(0);

			assertThat(file.getId()).isNotNull().hasSize(36);
			assertThat(file.getVendorId()).isEqualTo("assistants.52D79E5B2118D1740045AB87151535DCAD24E9A7");
			assertThat(file.getName()).isEqualTo("Helgetext2.odt");
			assertThat(file.getContentType()).isEqualTo("application/vnd.oasis.opendocument.text");
			assertThat(file.getSize()).isEqualTo(7993);
		}
	}

	@Nested
	class TestEinfachesFormularMitZweiAnhaengenVerschachtelt {

		private static final String GROUP1NAME = "datei_ausweis";
		private static final String GROUP2NAME = "datei_meldebestaetigung";

		List<IncomingFileGroup> attachments;

		@BeforeEach
		void init() throws ParserConfigurationException, SAXException, IOException {

			String xml = TestUtils.loadTextFile("intelliform/EinfachesFormularZweiAnhaengeXmlDatenVerschachtelt.xml");

			document = buildDocument(xml);

			attachments = mapper.mapAttachments(document);
		}

		@Test
		void validateAttachmentGroupCount() {

			assertThat(attachments).hasSize(2);
		}

		@Test
		void validateGroup1Name() {

			assertThat(attachments.get(0).getName()).isEqualTo(GROUP1NAME);
		}

		@Test
		void validateGroup1FilesCount() {

			assertThat(attachments.get(0).getFiles()).hasSize(2);
		}

		@Test
		void validateGroup1File1() {

			IncomingFile file = attachments.get(0).getFiles().get(0);

			assertThat(file.getId()).isNotNull().hasSize(36);
			assertThat(file.getVendorId()).isEqualTo("assistants.21B483DAA2DC7900C1D1135E566D0F672CB42832");
			assertThat(file.getName()).isEqualTo("20210326_133516[1].jpg");
			assertThat(file.getContentType()).isEqualTo("image/jpeg");
			assertThat(file.getSize()).isEqualTo(1184319);
		}

		@Test
		void validateGroup1File2() {

			IncomingFile file = attachments.get(0).getFiles().get(1);

			assertThat(file.getId()).isNotNull().hasSize(36);
			assertThat(file.getVendorId()).isEqualTo("assistants.2F1FF737CF4F23191C1952091CB342254EE22A37");
			assertThat(file.getName()).isEqualTo("20210326_133526[1].jpg");
			assertThat(file.getContentType()).isEqualTo("image/jpeg");
			assertThat(file.getSize()).isEqualTo(1384037);
		}

		@Test
		void validateGroup2Name() {

			assertThat(attachments.get(1).getName()).isEqualTo(GROUP2NAME);
		}

		@Test
		void validateGroup2FilesCount() {

			assertThat(attachments.get(1).getFiles()).hasSize(1);
		}

		@Test
		void validateGroup2File() {

			IncomingFile file = attachments.get(1).getFiles().get(0);

			assertThat(file.getId()).isNotNull().hasSize(36);
			assertThat(file.getVendorId()).isEqualTo("assistants.B20451156C677116B91A4EF2D1E859837B6CE510");
			assertThat(file.getName()).isEqualTo("20210324_103054[1].jpg");
			assertThat(file.getContentType()).isEqualTo("image/jpeg");
			assertThat(file.getSize()).isEqualTo(3066416);
		}
	}

	private Document buildDocument(String xml) throws ParserConfigurationException, SAXException, IOException {

		DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();

		docBuilderFactory.setFeature("http://apache.org/xml/features/disallow-doctype-decl", true);

		DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
		Document document = docBuilder.parse(new ByteArrayInputStream(xml.getBytes()));

		// optional, but recommended
		// http://stackoverflow.com/questions/13786607/normalization-in-dom-parsing-with-java-how-does-it-work
		document.getDocumentElement().normalize();

		return document;
	}

}
